
package com.ntut.csie.game;

import com.ntut.csie.game.gameState.GameState;
import com.ntut.csie.game.gameState.StateReady;
import com.ntut.csie.game.gameState.StateRun;
import com.ntut.csie.game.gameState.StateStop;

import android.content.Context;
import android.graphics.Canvas;
import android.hardware.SensorListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ViewFlipper;

public class GameView extends SurfaceView implements SurfaceHolder.Callback , SensorListener{
	
	class GameThread extends Thread {
		/** Message handler used by thread to interact with TextView */
        private Handler mHandler;
        /** Handle to the surface manager object we interact with */
        private SurfaceHolder mSurfaceHolder;
        /** Indicate whether the surface has been created & is ready to draw */
        private boolean mRun = false;
        private final int mCycleTime = 50;
        
        public GameState mGameState;	// the state instance 
        public int mState = STATE_READY;
        
        public GameThread(SurfaceHolder surfaceHolder, Context context, Handler handler) {
            mSurfaceHolder = surfaceHolder;
            mHandler = handler;
            mContext = context;
            
            mGameState = new StateReady();
            mGameState.Initialize();
        }
        
        public int getGameState() {
            synchronized (mSurfaceHolder) {
                return mState;
            }
        }
        
        public void setGameState(int mode) {
            synchronized (mSurfaceHolder) {
                if (mState != mode) {
                	mState = mode;
                	
                	if(mGameState != null) {
                		mGameState.Destroy();
                    	mGameState = null;
                	}
                	
                    switch(mState){
                    case STATE_READY:
                    	mGameState = new StateReady();
                    	break;
                    case STATE_RUNNING:
                    	mGameState = new StateRun();
                    	Message msg = mHandler.obtainMessage();
                    	Bundle b = new Bundle();
                    	b.putString("state", "STATE_RUN");
                    	msg.setData(b);
                    	mHandler.sendMessage(msg);
                    	break;
                    case STATE_STOP:
                    	mGameState = new StateStop();
                    	//this is how we send data back up to the main GameView thread.
                        //if you look in constructor of GameView you will see code for
                        //Handling of messages. This is borrowed directly from lunar lander.
                    	Message msg2 = mHandler.obtainMessage();
                    	Bundle b2 = new Bundle();
                    	b2.putString("state", "STATE_STOP");
                    	msg2.setData(b2);
                    	mHandler.sendMessage(msg2);
                    	break;
                    default:
                    	break;
                    }
                    
                    mGameState.Initialize();
                }
            }
        }
        
        public void setRunning(boolean b) {
            mRun = b;
        }
        
        @Override
        public void run() {
            // while running do stuff in this loop...bzzz!
            while (mRun) {
            	long lastTime = System.currentTimeMillis();
                mCanvas = null;
                try {
                	mCanvas = mSurfaceHolder.lockCanvas(null);
                    synchronized (mSurfaceHolder) {                    	
                    	if(mGameState != null) {
                    		if (!mGameState.IsPaused())
                    			mGameState.OnMove();
                    		mGameState.OnShow();
                    	}
                    }
                } finally {
                    // do this in a finally so that if an exception is thrown
                    // during the above, we don't leave the Surface in an
                    // inconsistent state
                    if (mCanvas != null) {
                        mSurfaceHolder.unlockCanvasAndPost(mCanvas);
                    }
                }// end finally block
                long sleepTime = mCycleTime - (System.currentTimeMillis() - lastTime);
                if(sleepTime > 0) {
                	try {
    					Thread.sleep(sleepTime);
    				} catch (InterruptedException e) { }
              	}
            }// end while mrun block
        }
        
        public void pause() {
            synchronized (mSurfaceHolder) {
                mGameState.Pause();
            }
        }
        
        public void unpause() {
        	synchronized (mSurfaceHolder) {
                mGameState.Unpause();
            }
        }
        
        public void destory() {
        	synchronized (mSurfaceHolder) {
        		setRunning(false);
        		mGameState.Destroy();
        		mGameState = null;
        		System.gc();
            }
        }
        
        public void setSurfaceSize(int width, int height) {
            // synchronized to make sure these all change atomically
            synchronized (mSurfaceHolder) { }
        }
        
        public void OnTouchDown(int x, int y) {
        	if(mGameState != null)
        		mGameState.OnTouchDown(x, y);
        }
        
    	public void OnTouchMove(int x, int y) {
    		if(mGameState != null)
    			mGameState.OnTouchMove(x, y);
    	}
    	
    	public void OnTouchUp(int x, int y) {
    		if(mGameState != null)
    			mGameState.OnTouchUp(x, y);
    	}
    	
    	public void OnDeviceAzimuth(float degrees) {
    		if(mGameState != null)
    			mGameState.OnDeviceAzimuth(degrees);
    	}
    	
    	public void OnDevicePitch(float degrees) {
    		if(mGameState != null)
    			mGameState.OnDevicePitch(degrees);
    	}
    	
    	public void OnDeviceRoll(float degrees) {
    		if(mGameState != null)
    			mGameState.OnDeviceRoll(degrees);
    	}
    	
    	public void OnDeviceAccelerateX(float force) {
    		if(mGameState != null)
    			mGameState.OnDeviceAccelerateX(force);
    	}
    	
    	public void OnDeviceAccelerateY(float force) {
    		if(mGameState != null)
    			mGameState.OnDeviceAccelerateY(force);
    	}
    	
    	public void OnDeviceAccelerateZ(float force) {
    		if(mGameState != null)
    			mGameState.OnDeviceAccelerateZ(force);
    	}
    	
        public void KeyDown(int keyCode) {
        	if(mGameState != null)
        		mGameState.KeyDown(keyCode);
        }
        
        public void KeyUp(int keyCode) {
        	if(mGameState != null)
        		mGameState.KeyUp(keyCode);
        }
	}
	
	//------------------------GameView content------------------------------//
	public static int Width;
	public static int Height;
	public static final int STATE_READY = -1;
    public static final int STATE_RUNNING = 0;
    public static final int STATE_STOP = 1;
    public static final int KEY_UP = KeyEvent.KEYCODE_DPAD_UP;
    public static final int KEY_DOWN = KeyEvent.KEYCODE_DPAD_DOWN;
    public static final int KEY_RIGHT = KeyEvent.KEYCODE_DPAD_RIGHT;
    public static final int KEY_LEFT = KeyEvent.KEYCODE_DPAD_LEFT;
    public static final int KEY_SELECT = KeyEvent.KEYCODE_DPAD_CENTER;
	public static Canvas mCanvas;
	public static Context mContext;
	private static GameThread mThread;
    
	private Button mStartButton;
    private Button mHelpButton;
    private Button mAboutButton;
    private Button mExitButton;
    private Button mMenuButton;

	public GameView(Context context, AttributeSet attrs) {
		super(context, attrs);

		// register our interest in hearing about changes to our surface
		SurfaceHolder holder = getHolder();
		holder.addCallback(this);
		
		WindowManager mWinMgr = (WindowManager)context.getSystemService(Context.WINDOW_SERVICE);
		Width = mWinMgr.getDefaultDisplay().getWidth();
		Height = mWinMgr.getDefaultDisplay().getHeight();
		
		if (isInEditMode() == false) {
            mThread = new GameThread(holder, context, new Handler() {

                public void handleMessage(Message m) {
                	
                    if (m.getData().getString("state") == "STATE_STOP") {
                        mMenuButton.setVisibility(View.VISIBLE);
                    }
                    if (m.getData().getString("state") == "STATE_RUN") {
                    	mStartButton.setVisibility(View.INVISIBLE);
                        mHelpButton.setVisibility(View.INVISIBLE);
                        mAboutButton.setVisibility(View.INVISIBLE);
                        mExitButton.setVisibility(View.INVISIBLE);
                    }
                }//end handle msg
            });
            
        }

        setFocusable(true);
	}
	
	public static void SetState(int mode) {
		mThread.setGameState(mode);
	}
	
	public GameThread getThread() {
        return mThread;
    }
	
	public void SetStartButton(Button button) {
		mStartButton = button; // in order to set the visibility
    }
	
	public void SetHelpButton(Button button) {
		mHelpButton = button; // in order to set the visibility
    }
	
	public void SetAboutButton(Button button) {
		mAboutButton = button; // in order to set the visibility
    }
	
	public void SetExitButton(Button button) {
		mExitButton = button; // in order to set the visibility
    }
	
	public void SetMenuButton(Button button) {
        mMenuButton = button; // in order to set the visibility
    }
    
    @Override
    public void onWindowFocusChanged(boolean hasWindowFocus) {
        if (!hasWindowFocus) {
            if (mThread != null)
                mThread.pause();
        }
    }
    
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        mThread.setSurfaceSize(width, height);
    }

    public void surfaceCreated(SurfaceHolder arg0) {
        // start the thread here so that we don't busy-wait in run()
        // waiting for the surface to be created
        mThread.setRunning(true);
        mThread.start();
    }
    
    public void surfaceDestroyed(SurfaceHolder arg0) {
    	boolean retry = true;
        mThread.setRunning(false);
        while (retry) {
            try {
                mThread.join();
                retry = false;

            } catch (InterruptedException e) {
            }
        }
    }
    
    // when touching on screen (the surfaceView)
    @Override
    public boolean onTouchEvent(MotionEvent event) {
    	int x = (int)event.getX();
    	int y = (int)event.getY();
    	switch(event.getAction()) {
    		case MotionEvent.ACTION_DOWN:
    			mThread.OnTouchDown(x, y);
    			break;
    		case MotionEvent.ACTION_MOVE:
    			mThread.OnTouchMove(x, y);
    			break;
    		case MotionEvent.ACTION_UP:
    			mThread.OnTouchUp(x, y);
    			break;
    	}
    	return true;
    }
    
    @Override
	public void onSensorChanged(int sensor, float[] values) {
		if (mThread.getGameState() == STATE_RUNNING) {
			if(sensor == SensorManager.SENSOR_ORIENTATION) {
				mThread.OnDeviceAzimuth(values[0]);
				mThread.OnDevicePitch(values[1]);
				mThread.OnDeviceRoll(values[2]);
			}
			if(sensor == SensorManager.SENSOR_ACCELEROMETER) {
				mThread.OnDeviceAccelerateX(values[0]);
				mThread.OnDeviceAccelerateY(values[1]);
				mThread.OnDeviceAccelerateZ(values[2]);
			}
    	}
	}

	@Override
	public void onAccuracyChanged(int arg0, int arg1) {}
}
