/*
 * This class is available to play multiple sound at
 * the same time.
 * It is recommend to use in sound effects handling 
 * (ex: explosion, bullets fired sound effects...);
 * but not in a large media file (not a long music).
 * (ex: background music...)
 */

package com.ntut.csie.game.core;

import java.io.IOException;
import java.util.HashMap;

import com.ntut.csie.game.GameView;

import android.media.AudioManager;
import android.media.SoundPool;

public class SoundEffects {
	private SoundPool mSoundPool;  
	private HashMap<Integer, Integer> mSoundPoolMap;
	
	public SoundEffects() {
		mSoundPool = new SoundPool(10, AudioManager.STREAM_MUSIC, 100);
		mSoundPoolMap = new HashMap<Integer, Integer>();
	}
	  
	public void SetResource(int soundId, int resId) {
	    mSoundPoolMap.put(soundId, mSoundPool.load(GameView.mContext, resId, 1));
	    // Maps the specified key to the specified value.
	}
	public void SetResource(int soundId, String fileName) {
	    try {
			mSoundPoolMap.put(soundId, mSoundPool.load(GameView.mContext.getAssets().openFd(fileName), 1));
			// Maps the specified key to the specified value.
	    } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	  
	public int Play(int soundId) {  
	    return mSoundPool.play(mSoundPoolMap.get(soundId), 0.8f, 0.8f, 1, 0, 1.0f);  
	}
	
	//	the following arguments 'playId' is returned by calling function 'Play()'
	public void Stop(int playId) {
		mSoundPool.stop(playId);
	}
	
	public void Pause(int playId) {
		mSoundPool.pause(playId);
	}
	
	public void Resume(int playId) {
		mSoundPool.resume(playId);
	}
	
	public void Release() {
		mSoundPoolMap.clear();
		mSoundPoolMap = null;
		mSoundPool.release();
		mSoundPool = null;
	}
	
	public class SoundId {
		private int mSoundId;
		private int mLoadId;
		private int mPlayId;
		
		public void SetSoundId(int id) {
			mSoundId = id;
		}
		
		public void SetLoadId(int id) {
			mLoadId = id;
		}
		
		public void SetPlayId(int id) {
			mPlayId = id;
		}
		
		public int GetSoundId() {
			return mSoundId;
		}
		
		public int GetLoadId() {
			return mLoadId;
		}
		
		public int GetPlayId() {
			return mPlayId;
		}
	}
}
