package com.ntut.csie.game.gameState;

import android.graphics.Point;

import com.ntut.csie.game.GameView;
import com.ntut.csie.game.core.MovingBitmap;

public abstract class GameState {
	public static final int STATE_READY = GameView.STATE_READY;
    public static final int STATE_RUNNING = GameView.STATE_RUNNING;
    public static final int STATE_STOP = GameView.STATE_STOP;
    public static final int KEY_UP = GameView.KEY_UP;
    public static final int KEY_DOWN = GameView.KEY_DOWN;
    public static final int KEY_LEFT = GameView.KEY_LEFT;
    public static final int KEY_RIGHT = GameView.KEY_RIGHT;
    public static final int KEY_SELECT = GameView.KEY_SELECT;
    public static final int WIDTH = GameView.Width;
    public static final int HEIGHT = GameView.Height;
    public static final int STAGE_NUMBER = 2;
    public static int mStageIndex;	// now playing stage
	protected boolean isPaused = false;
	protected static boolean isShowKeyboard = false;
    
    protected boolean isMovingUp;
    protected boolean isMovingDown;
    protected boolean isMovingLeft;
    protected boolean isMovingRight;
    protected boolean isSelecting;
    
    private MovingBitmap FireOn;
	private MovingBitmap FireOff;
	private MovingBitmap UpOn;
	private MovingBitmap UpOff;
	private MovingBitmap RightOn;
	private MovingBitmap RightOff;
	private MovingBitmap LeftOn;
	private MovingBitmap LeftOff;
	private MovingBitmap DownOn;
	private MovingBitmap DownOff;
	private MovingBitmap BlackBackGround;
	private Point firecenterPoint = new Point();
	private Point upcenterPoint = new Point();
	private Point downcenterPoint = new Point();
	private Point rightcenterPoint = new Point();
	private Point leftcenterPoint = new Point();
	
	public abstract void OnMove();
	
	public void Initialize() {
		isMovingUp = false;
        isMovingDown = false;
        isMovingLeft = false;
        isMovingRight = false;
		FireOff = new MovingBitmap("fire_off");
		FireOn = new MovingBitmap("fire_on");
		UpOff = new MovingBitmap("up_off");
		UpOn = new MovingBitmap("up_on");
		DownOff = new MovingBitmap("down_off");
		DownOn = new MovingBitmap("down_on");
		RightOff = new MovingBitmap("right_off");
		RightOn = new MovingBitmap("right_on");
		LeftOff = new MovingBitmap("left_off");
		LeftOn = new MovingBitmap("left_on");
		BlackBackGround = new MovingBitmap("black");
		FireOff.SetXY(324, 94);
		FireOn.SetXY(324, 94);
		UpOff.SetXY(324, 22);
		UpOn.SetXY(324, 22);
		DownOff.SetXY(324, 166);
		DownOn.SetXY(324, 166);
		RightOff.SetXY(396, 94);
		RightOn.SetXY(396, 94);
		LeftOff.SetXY(252, 94);
		LeftOn.SetXY(252, 94);
		BlackBackGround.SetXY(240, 0);
		firecenterPoint.x = FireOn.GetX() + (FireOn.GetWidth()/2);
		firecenterPoint.y = FireOn.GetY() + (FireOn.GetHeight()/2);
		upcenterPoint.x = UpOn.GetX() + (UpOn.GetWidth()/2);
		upcenterPoint.y = UpOn.GetY() + (UpOn.GetHeight()/2);
		downcenterPoint.x = DownOn.GetX() + (DownOn.GetWidth()/2);
		downcenterPoint.y = DownOn.GetY() + (DownOn.GetHeight()/2);
		rightcenterPoint.x = RightOn.GetX() + (RightOn.GetWidth()/2);
		rightcenterPoint.y = RightOn.GetY() + (RightOn.GetHeight()/2);
		leftcenterPoint.x = LeftOn.GetX() + (LeftOn.GetWidth()/2);
		leftcenterPoint.y = LeftOn.GetY() + (LeftOn.GetHeight()/2);
	}
	
	public void OnShow() {
		if(isShowKeyboard) {
			FireOn.OnShow();
			UpOn.OnShow();
			DownOn.OnShow();
			RightOn.OnShow();
			LeftOn.OnShow();
		}
	}
	
	public abstract void Destroy();
	
	public void Pause() {
		isPaused = true;
	}
	
	public void Unpause() {
		isPaused = false;
	}
		
	public boolean IsPaused() {
		return isPaused;
	}
	
	public void GoToState(int mode) {
		GameView.SetState(mode);
	}
	
	public void KeyboardSwitch() {
		isShowKeyboard = !isShowKeyboard;
	}
	
	private void ButtonTouchDown(Point point, int x, int y, int key){
		double distance = getDistance(point, x, y);
		if(distance < 36)
			KeyDown(key);
	}
	
	private double getDistance(Point point, int x, int y){
		return Math.sqrt(((point.x-x)*(point.x-x)) + (point.y-y)*(point.y-y));
	}
	
	// touch event handler
	public void OnTouchDown(int x, int y) {
		ButtonTouchDown(firecenterPoint, x, y, KEY_SELECT);
		ButtonTouchDown(upcenterPoint, x, y, KEY_UP);
		ButtonTouchDown(downcenterPoint, x, y, KEY_DOWN);
		ButtonTouchDown(rightcenterPoint, x, y, KEY_RIGHT);
		ButtonTouchDown(leftcenterPoint, x, y, KEY_LEFT);
	}
	public void OnTouchMove(int x, int y) {}
	
	public void OnTouchUp(int x, int y) {
		KeyUp(KEY_SELECT);
		KeyUp(KEY_UP);
		KeyUp(KEY_DOWN);
		KeyUp(KEY_RIGHT);
		KeyUp(KEY_LEFT);
	}
	
	// device rotate event handler
	public void OnDeviceAzimuth(float degrees) {}
	public void OnDevicePitch(float degrees) {}
	public void OnDeviceRoll(float degrees) {}
	
	// device accelerate event handler
	public void OnDeviceAccelerateX(float force) {}
	public void OnDeviceAccelerateY(float force) {}
	public void OnDeviceAccelerateZ(float force) {}
	
	// key event handler
	public void KeyDown(int keyCode) {
		if(isShowKeyboard) {
			if(keyCode == KEY_UP) {
	            isMovingUp = true;
	        }
	        if(keyCode == KEY_DOWN) {
	            isMovingDown = true;
	        }
	        if(keyCode == KEY_LEFT) {
	            isMovingLeft = true;
	        }
	        if(keyCode == KEY_RIGHT) {
	            isMovingRight = true;
	        }
	        if(keyCode == KEY_SELECT) {
	            isSelecting = true;
	        }
		}	
	}
	
	public void KeyUp(int keyCode) {
		if(isShowKeyboard) {
			if(keyCode == KEY_UP) {
	            isMovingUp = false;
	        }
	        if(keyCode == KEY_DOWN) {
	            isMovingDown = false;
	        }
	        if(keyCode == KEY_LEFT) {
	            isMovingLeft = false;
	        }
	        if(keyCode == KEY_RIGHT) {
	            isMovingRight = false;
	        }
	        if(keyCode == KEY_SELECT) {
	            isSelecting = false;
	        }
		}
	}
}
